﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Core;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using System.Collections.Generic;
using HeuristicLab.Problems.VehicleRouting.Encodings.General;
using HeuristicLab.Problems.VehicleRouting.Interfaces;
using HeuristicLab.Common;
using HeuristicLab.Problems.VehicleRouting.Encodings.Potvin;

namespace HeuristicLab.Problems.VehicleRouting.Encodings.ExtendedPotvin {
  [Item("ExtendedPotvinCrossover", "A VRP crossover operation.")]
  [StorableClass]
  public class ExtendedPotvinCrossover : VRPCrossover, IStochasticOperator, IExtendedPotvinOperator {
    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }

    public IValueParameter<PotvinCrossover> ToursCrossover {
      get { return (IValueParameter<PotvinCrossover>)Parameters["ToursCrossover"]; }
    }

    [StorableConstructor]
    protected ExtendedPotvinCrossover(bool deserializing) : base(deserializing) { }

    public ExtendedPotvinCrossover() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used for stochastic manipulation operators."));
      Parameters.Add(new ValueParameter<PotvinCrossover>("ToursCrossover",
        "The operator used to crossover the tours.", new PotvinInsertionBasedCrossover()));
    }

    protected ExtendedPotvinCrossover(ExtendedPotvinCrossover original, Cloner cloner)
      : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ExtendedPotvinCrossover(this, cloner);
    }

    public override IOperation Apply() {
      ItemArray<IVRPEncoding> parents = new ItemArray<IVRPEncoding>(ParentsParameter.ActualValue.Length);
      for (int i = 0; i < ParentsParameter.ActualValue.Length; i++) {
        IVRPEncoding solution = ParentsParameter.ActualValue[i];

        if (!(solution is ExtendedPotvinEncoding)) {
          parents[i] = ExtendedPotvinEncoding.ConvertFrom(solution, ProblemInstance);
        } else {
          parents[i] = solution;
        }
      }
      ParentsParameter.ActualValue = parents;

      OperationCollection next = new OperationCollection(base.Apply());
      
      ToursCrossover.Value.ParentsParameter.ActualName = ParentsParameter.ActualName;
      ToursCrossover.Value.ChildParameter.ActualName = ChildParameter.ActualName;
      next.Insert(0, ExecutionContext.CreateOperation(ToursCrossover.Value));

      return next;
    }
  }
}
