﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System.Drawing;
using System.Collections.Generic;
using HeuristicLab.Problems.VehicleRouting.Encodings.General;
using HeuristicLab.Problems.VehicleRouting.Variants;
using HeuristicLab.Problems.VehicleRouting.Interfaces;
using HeuristicLab.Problems.VehicleRouting.ProblemInstances;
using HeuristicLab.Problems.VehicleRouting.Encodings.Potvin;

namespace HeuristicLab.Problems.VehicleRouting.Encodings.ExtendedPotvin {
  [Item("ExtendedPotvinEncoding", "A tour encoding with heterogenous vehicles.")]
  [StorableClass]
  public class ExtendedPotvinEncoding : PotvinEncoding {
    [Storable]
    public Permutation VehicleAssignment { get; private set; }

    public ExtendedPotvinEncoding(IVRPProblemInstance instance)
      : base(instance) {
       VehicleAssignment = new Permutation(PermutationTypes.Absolute, instance.Vehicles.Value);
    }

    [StorableConstructor]
    private ExtendedPotvinEncoding(bool serializing)
      : base(serializing) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ExtendedPotvinEncoding(this, cloner);
    }

    protected ExtendedPotvinEncoding(ExtendedPotvinEncoding original, Cloner cloner)
      : base(original, cloner) {
        this.VehicleAssignment = cloner.Clone<Permutation>(original.VehicleAssignment);
    }

    public override int GetVehicleAssignment(int tour) {
      return VehicleAssignment[tour];
    }

    public static new ExtendedPotvinEncoding ConvertFrom(IVRPEncoding encoding, IVRPProblemInstance instance) {
      ExtendedPotvinEncoding solution = new ExtendedPotvinEncoding(instance);

      TourEncoding.ConvertFrom(encoding, solution, instance);

      List<int> vehicles = new List<int>();
      for (int i = 0; i < instance.Vehicles.Value; i++)
        vehicles.Add(i);

      int[] assignment = new int[instance.Vehicles.Value];
      for (int i = 0; i < assignment.Length; i++)
        assignment[i] = -1;

      for (int i = 0; i < solution.Tours.Count; i++) {
        int vehicle = encoding.GetVehicleAssignment(i);
        assignment[i] = vehicle;
        vehicles.Remove(vehicle);
      }

      for (int i = 0; i < instance.Vehicles.Value; i++) {
        if (assignment[i] == -1) {
          int vehicle = vehicles[0];
          assignment[i] = vehicle;
          vehicles.RemoveAt(0);
        }
      }

      solution.VehicleAssignment = new Permutation(PermutationTypes.Absolute, assignment);

      return solution;
    }

    public static new ExtendedPotvinEncoding ConvertFrom(List<int> route, IVRPProblemInstance instance) {
      ExtendedPotvinEncoding solution = new ExtendedPotvinEncoding(instance);

      TourEncoding.ConvertFrom(route, solution);

      return solution;
    }
  }
}
